#ifndef _CAMERA_
#define _CAMERA_

#include "gmtl/gmtl.h"

class Camera {
	public:		
		Camera(float l, float r, float b, float t, float n, float f);
		Camera(float fovy, float aspect, float n, float f);
		Camera();
				
		void setLocation(float posx, float posy, float posz, 
						 float centerx, float centery, float centerz,	
						 float upx, float upy, float upz);
		void setOrbit(float rotx, float roty, float trans);
		
		void setPosition(float posx, float posy, float posz);
		void setCenter(float centerx, float centery, float centerz);
		void setUp(float upx, float upy, float upz);
				
		void setProjection(float l, float r, float b, float t, float n, float f);
		void setProjection(float fovy, float aspect, float n, float f);

		void setFarPlaneDistance(float fDist);
		
		gmtl::Vec3f getPosition();
		gmtl::Vec3f getCenter();
		gmtl::Vec3f getUp();
		gmtl::Rayf getLineOfSight() const;
		
		float getLeftDistance() const;
		float getRightDistance() const;
		float getTopDistance() const;
		float getBottomDistance() const;
		float getNearDistance() const;
		float getFarDistance() const;

		float Camera::getFovy() const;
		
		gmtl::Matrix44f m_C;
		gmtl::Matrix44f m_P;
		gmtl::Vec4f m_vtVertices[8];
		gmtl::Vec4f m_vtVerticesTransformed[8];
	private:
		void Frustum();
		void LookAt();

		float fLeft, fRight, fBottom, fTop, fNear, fFar;
		float fovy;

		gmtl::Vec3f m_vecPosition;
		gmtl::Vec3f m_vecCenter;
		gmtl::Vec3f m_vecUp;
		
		gmtl::Rayf m_rayLineOfSight;

		bool m_bProjectionInit;
};


inline gmtl::Vec3f Camera::getPosition() {
	return m_vecPosition;
}

inline gmtl::Vec3f Camera::getCenter() {
	return m_vecCenter;
}

inline gmtl::Vec3f Camera::getUp() {
	return m_vecUp;
}

inline void Camera::setPosition(float posx, float posy, float posz) {
	setLocation(posx, posy, posz,
				m_vecCenter[0], m_vecCenter[1], m_vecCenter[2],
				m_vecUp[0], m_vecUp[1], m_vecUp[2]);
}

inline void Camera::setCenter(float centerx, float centery, float centerz) {
	setLocation(m_vecPosition[0], m_vecPosition[1], m_vecPosition[2],
				centerx, centery, centerz,
				m_vecUp[0], m_vecUp[1], m_vecUp[2]);
}

inline void Camera::setUp(float upx, float upy, float upz) {
	setLocation(m_vecPosition[0], m_vecPosition[1], m_vecPosition[2],
				m_vecCenter[0], m_vecCenter[1], m_vecCenter[2],
				upx, upy, upz);
}

inline void Camera::setFarPlaneDistance(float fDist) {
	setProjection(fLeft, fRight, fBottom, fTop, fNear, fDist);
}

inline float Camera::getFarDistance() const {
	return fFar;
}

inline float Camera::getNearDistance() const {
	return fNear;
}

inline float Camera::getLeftDistance() const {
	return fLeft;
}

inline float Camera::getRightDistance() const {
	return fRight;
}

inline float Camera::getBottomDistance() const {
	return fBottom;
}

inline float Camera::getTopDistance() const {
	return fTop;
}

inline float Camera::getFovy() const {
	return fovy;
}

inline gmtl::Rayf Camera::getLineOfSight() const {
	return m_rayLineOfSight;
}

#endif //_CAMERA_
