#include "Line2D.h"
#include <math.h>

using namespace std;
using namespace gmtl;

Line2D Line2D::ortho() {
	Vec2f u = b - a;
	Vec2f v(-u[1], u[0]);
	return Line2D(a, v + a);
}

Line2D Line2D::parallel(gmtl::Vec2f& q) {
	Vec2f u = b - a;
	Vec2f p = q + u;
	return Line2D(q, p);
}

Vec2f Line2D::maxPointToLine(std::vector<gmtl::Vec2f>& v) {
	float fMax = 0.0f, f;
	int iMax = 0;
	
	for(int i = 0; i < v.size(); i++) {
		f = fabs(distpoint(v[i]));
		if(f > fMax)  {
			fMax = f;
			iMax = i;
		}
	}
	return v[iMax];
}

float Line2D::distpoint(gmtl::Vec2f& p) const {
	return ((b[0] - a[0]) * (a[1] - p[1]) - (a[0] - p[0]) * (b[1] - a[1])) / sqrt((b[0] - a[0]) * (b[0] - a[0]) + (b[1] - a[1]) * (b[1] - a[1]));
}

bool intersect(gmtl::Vec2f& i, const Line2D& l1, const Line2D& l2) {
	float denominator = (l1.b[1] * (l2.a[0] - l2.b[0]) + l1.a[1] * (l2.b[0] - l2.a[0]) + (l1.a[0] - l1.b[0]) * (l2.a[1] - l2.b[1]));

	//if(denominator < 0.001)
	//	return false;

	i = Vec2f((-l1.b[0] * l2.a[1] * l2.b[0] + l1.a[1] * l1.b[0] * (l2.b[0] - l2.a[0]) + l1.b[0] * l2.a[0] * l2.b[1] + l1.a[0] * (l1.b[1] * l2.a[0] - l2.b[1] * l2.a[0] - l1.b[1] * l2.b[0] + l2.a[1] * l2.b[0])) / denominator,
			  ( l1.b[1] * (-l2.a[1] * l2.b[0] + l1.a[0] * (l2.a[1] - l2.b[1]) + l2.a[0] * l2.b[1]) + l1.a[1] * (l2.a[1] * l2.b[0] - l2.a[0] * l2.b[1] + l1.b[0] * (l2.b[1] - l2.a[1]))) / denominator);

	return true;
}